@@ -7,7 +7,7 @@ use strict;
 use warnings;
 use Carp 'croak';
 
-our $VERSION = '0.2';
+our $VERSION = '1.0';
 
 
 # Store the object in a global variable for some functions that don't get it
@@ -232,7 +232,7 @@ sub _handle_request {
     return 1 if $self->{_TUWF}{pre_request_handler} && !$self->{_TUWF}{pre_request_handler}->($self);
 
     # find the handler
-    my $loc = $self->reqPath;
+    (my $loc = $self->reqPath) =~ s/^\///;
     study $loc;
     my $han = $self->{_TUWF}{error_404_handler};
     my @args;
@@ -281,7 +281,7 @@ sub _handle_request {
       "HTTP Request Headers:\n".
       join('', map sprintf("  %s: %s\n", $_, $self->reqHeader($_)), $self->reqHeader).
       "POST dump:\n".
-      join('', map sprintf("  %s: %s\n", $_, $self->reqPost($_)), $self->reqPost).
+      join('', map sprintf("  %s: %s\n", $_, join "\n    ", $self->reqPosts($_)), $self->reqPosts).
       "Error:\n  $err\n"
     );
   }
@@ -300,7 +300,7 @@ sub _handle_request {
       $sqlt += $_->[1]*1000 for (@{$self->{_TUWF}{DB}{queries}});
     }
 
-    $self->log(sprintf('%4dms (SQL:%4dms,%3d qs)', $time, $sqlt, $sqlc, $self->reqURI));
+    $self->log(sprintf('%4dms (SQL:%4dms,%3d qs)', $time, $sqlt, $sqlc));
   }
 }
 
